@@ -49,6 +49,8 @@ Options:
   --silent                    Show only the progression of repetitions instead
                               of the progression of each simulation.
   --quiet                     Show no progression information at all.
+  --no-count                  Disable the production of file counts.csv (hence
+                              accelerates simulation)
   --save FILE                 Save simulation state (all agents state and
                               parameters) at the end of the simulation.
   --load FILE                 Use a saved simulation to start the current one.
@@ -63,6 +65,8 @@ Options:
                               option -p/--param.
   --format FORMAT             Specify an image format for diagram outputs
                               [default: svg].
+  --table-params              Display a table of the parameters and initial
+                              conditions.
 
 Advanced options:
   --seed SEED                 Set a seed value for random numbers. When not
@@ -95,14 +99,15 @@ EMULSION (Epidemiological Multi-Level Simulation framework)
 Contributors and contact:
 -------------------------
 
-    - Sébastien Picault (sebastien.picault@inrae.fr)
+    - Sébastien Picault (sebastien.picault@inra.fr)
     - Yu-Lin Huang
     - Vianney Sicard
     - Sandie Arnoux
     - Gaël Beaunée
-    - Pauline Ezanno (pauline.ezanno@inrae.fr)
+    - Pauline Ezanno (pauline.ezanno@inra.fr)
 
-    INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+    BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+    Nantes CS 44307 CEDEX, France
 
 
 How to cite:
@@ -118,24 +123,24 @@ How to cite:
 License:
 --------
 
-   Copyright 2016 INRAE and Univ. Lille
+    Copyright 2016 INRAE and Univ. Lille
 
-   Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 
-   Agence pour la Protection des Programmes,
-   54 rue de Paradis, 75010 Paris, France
+    Agence pour la Protection des Programmes,
+    54 rue de Paradis, 75010 Paris, France
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-       http://www.apache.org/licenses/LICENSE-2.0
+        http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
 ## TO BE IMPLEMENTED LATER:
@@ -175,13 +180,13 @@ from   textx.export                 import metamodel_export, model_export
 import colorama
 
 import emulsion
-from   emulsion.model               import EmulsionModel
+from   emulsion.model.emulsion_model import EmulsionModel
 from   emulsion.tools.state         import StateVarDict
 from   emulsion.tools.misc          import load_class
 from   emulsion.tools.plot          import plot_outputs
 from   emulsion.tools.simulation    import MultiSimulation
 
-VERSION = '1.0.12'
+VERSION = '1.1.0'
 DEFAULT_VERSION = "1.0"
 LICENSE = 'Apache-2.0'
 DEFAULT_LICENSE = "Apache-2.0"
@@ -338,7 +343,8 @@ def run_model(params):
     multi_simu.run()
     end = time.perf_counter()
     print(colorama.Style.BRIGHT + 'Simulation finished in {:.2f} s'.format(end-start))
-    print(colorama.Fore.GREEN + 'Outputs stored in {}'.format(count_path) + colorama.Style.RESET_ALL)
+    if not params.nocount:
+        print(colorama.Fore.GREEN + 'Outputs stored in {}'.format(count_path) + colorama.Style.RESET_ALL)
     return multi_simu
 
 
@@ -461,6 +467,52 @@ def init_main_level(params):
               colorama.Style.RESET_ALL)
         sys.exit(-1)
 
+
+def table_param(params):
+    """Display a table of parameters and initial conditions.
+
+    """
+    parameters = params.model._description['parameters']
+    strOutput = 'PARAMETERS\n__________\n\n'
+    strOutput += '|   name                 |    value '+(' '*41)+'| description\n'
+    strOutput += '-' * 200 + '\n'
+    for p in parameters:
+        if '{' not in p:
+            strOutput += create_desc(parameters, p)
+    print(strOutput)
+
+    initials = params.model._description['initial_conditions']
+    strOutput = '\n\nINITIAL CONDITIONS\n__________________\n\n'
+    strOutput += '|   name                 |    value\n'
+    strOutput += '-' * 200 + '\n'
+    strOutput += create_initial(initials)
+    print(strOutput)
+
+def create_desc(parameters, p):
+    name = p
+    result = '| ' + name + ' '*(23 - len(name)) + '| '
+
+    val = str(parameters[p]['value'])
+    result += val + ' '*(50 - len(val)) + '| '
+
+    desc = parameters[p]['desc']
+    result += desc+'\n'
+
+    return result
+
+def create_initial(initials):
+    result = ''
+    for i in initials:
+        result += colorama.Style.DIM + '  ' + i + '\n' + colorama.Style.RESET_ALL
+        desc = initials[i]
+        for elem in desc:
+            for e in elem:
+                result += '  ' + e + ' '*(23 - len(e))
+                result += '| ' + str(elem[e]) + '\n'
+            result += '\n'
+    return result
+
+
 def main(args=None):
     """Run EMuLSion's main program according to the command-line
     arguments.
@@ -480,7 +532,8 @@ def main(args=None):
         sys.exit(-1)
 
     params = StateVarDict()
-    params.model = EmulsionModel(filename=args['MODEL'])
+    params.model = EmulsionModel(filename=args['MODEL'],
+                                 input_dir=Path(args['--input-dir']))
     params.nb_simu = int(args['--runs'])
     params.stochastic = not args['--deterministic']
     params.to_describe = args['PARAM']
@@ -496,9 +549,9 @@ def main(args=None):
 
     params.silent = args['--silent']
     params.quiet = args['--quiet']
+    params.nocount = args['--no-count']
     params.log_params = args['--log-params']
     params.start_id = int(args['--start-id'])
-
     params.output_dir = Path(args['--output-dir'])
     params.input_dir = Path(args['--input-dir'])
     params.figure_dir = Path(args['--figure-dir'])
@@ -537,6 +590,9 @@ def main(args=None):
         produce_diagrams(params)
         params.view_machines = True
 
+    if args['--table-params']:
+        table_param(params)
+
     if args['check']:
         check_model(params, args['MODEL'], args['--meta-tree'], args['--model-tree'])
     elif args['diagrams']:
