@@ -11,14 +11,15 @@ Classes and functions for actions.
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -34,24 +35,24 @@ Classes and functions for actions.
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 from   abc                 import abstractmethod
 import numpy               as     np
@@ -151,7 +152,7 @@ class RecordChangeAction(ValueAction):
 
         """
         if population is None:
-            population = len(agents)
+            population = len(agents) if agents is not None else 0
         target_agent = unit.upper_level()
         value = target_agent.statevars[self.statevar_name]\
                 if self.statevar_name in target_agent.statevars else 0
@@ -191,14 +192,63 @@ class SetVarAction(ValueAction):
             agents = [unit]
         for agent in agents:
             # if the value which has to be affected to the variable is
-            # already a statevar, retrieve its value from the agent
-            if self.parameter in agent.statevars:
-                value = agent.statevars[self.parameter]
-            # otherwise get its value through the model
-            else:
+            # a value defined in the model, get it
+            if self.parameter in agent.model._values:
                 value = agent.get_model_value(self.parameter)
+            # otherwise get its value through the agent
+            else:
+                value = agent.get_information(self.parameter)
             # agent.set_information(self.statevar_name, value)
-            agent.statevars[self.statevar_name] = value
+            if agent.level is None:
+                agent.upper_level().statevars[self.statevar_name] = value
+            else:
+                agent.statevars[self.statevar_name] = value
+
+    def __str__(self):
+        return super().__str__() + ' {} <- {}'.format(self.statevar_name,
+                                                      self.parameter)
+    __repr__ = __str__
+
+
+class SetUpperVarAction(ValueAction):
+    """SetUpperVarAction allows to set the variable of the upper level of the agent.
+
+    """
+    def __init__(self, statevar_name=None, parameter=None, model=None, **others):
+        """Create a SetUpperVarAction aimed at modifying the specified statevar
+        according to the parameter.
+
+        """
+        super().__init__(**others)
+        if statevar_name in model.state_machines:
+            raise InvalidActionException("Action set_var must not change values of state machines. Use action become instead.\n\tset_var: {} value: {}".format(statevar_name, parameter))
+        self.statevar_name = statevar_name
+        self.parameter = parameter
+        # print(self)
+
+    def execute_action(self, unit, agents=None, **others):
+        """Execute the action in the upper level of the specified unit. If the `agents` parameter
+        is specified (as a list), each agent of this list will execute
+        the action. If changes of state variables in relation to a
+        state machine occur, the corresponding actions (if any) are
+        executed: on_exit from the current state, and on_enter for the
+        new state.
+
+        """
+        # print('COUCOU')
+        if agents is None:
+            agents = [unit]
+        for agent in agents:
+            # if the value which has to be affected to the variable is
+            # a value defined in the model, get it
+            if self.parameter in agent.model._values:
+                value = agent.get_model_value(self.parameter)
+            # otherwise get its value through the agent
+            else:
+                value = agent.get_information(self.parameter)
+            # agent.set_information(self.statevar_name, value)
+            upper_level = agent.upper_level() if agent.level is not None else agent.upper_level().upper_level()
+            upper_level.statevars[self.statevar_name] = value
 
     def __str__(self):
         return super().__str__() + ' {} <- {}'.format(self.statevar_name,
@@ -365,18 +415,23 @@ class BecomeAction(AbstractAction):
         for agent in agents:
             protos = list(self.prototype_names)
             # compute actual values for probabilities
-            proba_values = [agent.get_model_value(prob) for prob in self.probas]
+            proba_values = [agent.get_model_value(prob)\
+                            if prob in agent.model._values else agent.get_information(prob)
+                            for prob in self.probas]
             total = sum(proba_values)
             assert(0 <= total <= 1)
-            if total < 1:
+            if len(proba_values) < len(protos):
                 # add complement
                 proba_values.append(1 - total)
                 # if N-1 probabilities were given for N prototypes, no
                 # problem: the last prototype is getting the 1-total
-                # value. Otherwise, this means that there is a
-                # possibility that no individuals are produced => None
-                if len(proba_values) > len(protos):
-                    protos.append(None)
+                # value.
+            elif total < 1:
+                # otherwise (N probas with N protos but total < 1),
+                # this means that there is a possibility that no
+                # individuals are produced => None
+                protos.append(None)
+                proba_values.append(1 - total)
             prototype = np.random.choice(protos, p=proba_values)
             if prototype is not None:
                 agent.apply_prototype(name=prototype, execute_actions=True)
@@ -484,7 +539,7 @@ class LogVarsAction(StringAction):
             agents = [unit]
         for agent in agents:
             message = '{},{},{},{}'.format(agent.get_information('simu_id'), agent.statevars.step, agent.__class__.__name__, agent.agid)
-            variables = ','.join(['{}={}'.format(varname, agent.get_information(varname))
+            variables = ';'.join(['{}={}'.format(varname, agent.get_information(varname))
                                   for varname in self.l_params])
             with open(agent.log_path(), 'a') as logfile:
                 logfile.write('{},{}\n'.format(message, variables))
@@ -516,7 +571,8 @@ class MethodAction(AbstractAction):
         if agents is None:
             agents = [unit]
         for agent in agents:
-            action = getattr(agent, self.method)
+            # if current agent is not a level, get action at upper level:
+            action = getattr(agent, self.method) if agent.level is not None else getattr(agent.upper_level(), self.method)
             l_params = [retrieve_value(self.state_machine.get_value(expr), agent)
                         for expr in self.l_params]
             ### introduced to pass internal information such as population
@@ -572,5 +628,6 @@ ACTION_DICT = {
     'action': MethodAction,
     'duration': FunctionAction,
     'set_var': SetVarAction,
+    'set_upper_var': SetUpperVarAction,
     'record_change': RecordChangeAction,
 }
