@@ -14,14 +14,15 @@ Univ. Lille).
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -37,24 +38,24 @@ Univ. Lille).
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 from   functools                  import total_ordering
 
@@ -133,6 +134,8 @@ class AbstractAgent(object, metaclass=MetaAgent):
         self._envt = envt
         # environment encapsulated by the agent
         self._content = content
+        # last prototype applied to the agent
+        self._last_prototype = None
         # simulation where the agent belongs
         self.simulation = None
         # model used to define simulation behaviors
@@ -424,12 +427,21 @@ class AbstractAgent(object, metaclass=MetaAgent):
                 self.statevars['_time_entered_{}'.format(machine_name)]) *\
             self.model.delta_t
 
+    def reapply_prototype(self, execute_actions=False):
+        """Reapply to self the last prototype that was used on this agent. The
+        whole definition (not the name) of the prototype to apply is
+        stored in attribute ``_last_prototype``. Use e.g. to apply a
+        prototype first without actions, then reapply it with actions.
+
+        """
+        self.apply_prototype(prototype=self._last_prototype, execute_actions=execute_actions)
+
     def apply_prototype(self, name=None, prototype=None, execute_actions=False):
         """Apply the prototype with the specified name to this agent. A
         prototype is basically a dictionary of statevars associated to
         values. Thus this method changes specific statevars to new
         values as required in the prototype. Other statevars are kept
-        unchanged.  If a statevar was no amongst those of this agent,
+        unchanged. If a statevar was no amongst those of this agent,
         it is added. When statevars hold the state of a state machine,
         the state is changed with correct initialization of
         corresponding attributes (mainly _time_entered) ; if
@@ -437,21 +449,45 @@ class AbstractAgent(object, metaclass=MetaAgent):
         performed.
 
         TAG: USER
+
         """
         # give priority to named prototypes if provided, else
         if name is not None:
-            prototype = self.model.get_prototype(self.level, name)
+            prototype = self.model.get_prototype(self.level, name, self.get_information('simu_id'))
         if prototype is None:
             print("WARNING: agent {} was asked to apply inexistent prototype".format(self))
+        self._last_prototype = prototype
+        # print(self, 'applying prototype', name, prototype, execute_actions)
+        # if any 'begin_with' sequence is defined, apply immediately in the specified order
+        if 'begin_with' in prototype:
+            l_tuples = prototype['begin_with']
+            for var, value in l_tuples:
+                val = value(self) if callable(value) else value
+                # set the value to the variable depending on the nature of the variable
+                if var in self.model.state_machines:
+                    self.change_state(var, val, do_actions=execute_actions)
+                else:
+                    self.statevars[var] = val
+        # for regular cases:
         # apply changes in states first, then other variables
         for var, value in prototype.items():
             if var in self.model.state_machines:
                 val = value(self) if callable(value) else value
                 self.change_state(var, val, do_actions=execute_actions)
         for var, value in prototype.items():
-            if var not in self.model.state_machines:
+            if var not in self.model.state_machines and var != 'in_order':
                 val = value(self) if callable(value) else value
                 self.statevars[var] = val
+        # if any 'end_with' sequence is defined, apply finally in the specified order
+        if 'end_with' in prototype:
+            l_tuples = prototype['end_with']
+            for var, value in l_tuples:
+                val = value(self) if callable(value) else value
+                # set the value to the variable depending on the nature of the variable
+                if var in self.model.state_machines:
+                    self.change_state(var, val, do_actions=execute_actions)
+                else:
+                    self.statevars[var] = val
 
         ### ADDED TO ENSURE CONSISTENCY WHEN APPLYING PROTOTYPES.
         ### TODO: revise prototype application to enhance efficiency
@@ -460,10 +496,11 @@ class AbstractAgent(object, metaclass=MetaAgent):
 
     def apply_initial_prototype(self, name=None, prototype=None, execute_actions=False):
         """Apply prototype to a newly created agent. This function is called
-        by `new_atom`in `MultiProcessManager` and redefined to do nothing in
+        by `new_atom` in `MultiProcessManager` and redefined to do nothing in
         this class, in order to ensure that the prototype of a level is
         applied before creating entities for the sublevels.
         """
+        # print(self, 'applying initial prototype', name, prototype, execute_actions)
         if name is not None:
             # print('applying prototype', name, 'on', self)
             self.apply_prototype(name=name, execute_actions=execute_actions)
