@@ -14,14 +14,15 @@ Univ. Lille).
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -37,24 +38,24 @@ Univ. Lille).
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 import abc
 
@@ -62,7 +63,7 @@ import numpy                      as np
 
 
 from   emulsion.tools.graph       import EdgeTypes
-from   emulsion.model             import EDGE_KEYWORDS
+from   emulsion.model.functions   import EDGE_KEYWORDS
 from   emulsion.tools.misc        import rates_to_probabilities,\
     aggregate_probabilities, probabilities_to_rates, count_population, AGENTS
 
@@ -96,6 +97,7 @@ class EmulsionAgent(AbstractAgent):
         """Initialize the unit with a health state and a name."""
         super().__init__(**others)
         # self.statevars.health_state = health_state
+        self.activate()
         self.model = model
         self.level = level
         self._name = name
@@ -107,6 +109,27 @@ class EmulsionAgent(AbstractAgent):
         # exchange inbox/outbox
         self._inbox = []
         self._outbox = []
+        if self.level is not None:
+            if self.level in self.model.default_prototypes:
+                self.apply_prototype(name=self.model.default_prototypes[self.level],
+                                     execute_actions=True)
+
+    def activate(self):
+        """Make this agent active, i.e. able to perform its processes"""
+        self.statevars._is_active = True
+
+    def deactivate(self):
+        """Make this agent inactive, i.e. unable to perform its processes"""
+        self.statevars._is_active = False
+
+    def detach_model(self):
+        """Remove the reference to the model in all agents. Required before
+        serialization.
+
+        """
+        self.model = None
+        self.simulation = None
+        self._last_prototype = None
 
     def upper_level(self, init=True):
         """Return the 'upper level' for this agent, i.e. the first host with
