@@ -12,14 +12,15 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -35,26 +36,32 @@
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 import abc
+import pickle as serializer
+# import cloudpickle as serializer
+# import dill as serializer
+# serializer.settings['recurse'] = True
+
+import time
 import itertools                 as it
 
 from   sortedcontainers          import SortedSet
@@ -65,10 +72,8 @@ import pandas                    as pd
 from   emulsion.agent.views      import StructuredView, AdaptiveView
 from   emulsion.agent.process    import MethodProcess, StateMachineProcess
 from   emulsion.agent.exceptions import StateVarNotFoundException
-from   emulsion.tools.misc       import load_class, add_new_property,\
-    create_population_getter, add_all_test_properties, create_aggregator,\
-    create_group_aggregator
-
+from   emulsion.tools.misc       import load_class, add_new_property, add_all_test_properties, add_all_relative_population_getters
+from   emulsion.tools.getters    import create_population_getter, create_aggregator, create_group_aggregator, make_information_getter
 
 from   emulsion.agent.managers.group_manager  import  GroupManager
 
@@ -98,7 +103,7 @@ class AbstractProcessManager(StructuredView):
     def __init__(self, model=None, master=None, level=None, stochastic=True,
                  keep_history=False, prototype=None, custom_prototype=None,
                  execute_actions=False, **others):
-        super().__init__(recursive=False, **others)
+        super().__init__(recursive=False, level=level, model=model, **others)
         self.statevars.population = 0
         self.stochastic = stochastic
         self.model = model
@@ -112,8 +117,7 @@ class AbstractProcessManager(StructuredView):
         # dynamically add property 'total_level' which returns the
         # total population in this level, and aggegate variables if any
         if self.level is not None:
-            add_new_property(self, 'total_{}'.format(self.level),
-                             lambda self: self.get_information('population'))
+            add_new_property(self, 'total_{}'.format(self.level), make_information_getter('population'))
             if self.level in self.model.aggregate_vars:
                 for (varname, sourcevar, operator) in self.model.aggregate_vars[self.level]:
                     add_new_property(self, varname, create_aggregator(sourcevar, operator))
@@ -126,12 +130,34 @@ class AbstractProcessManager(StructuredView):
         self.apply_initial_conditions()
         self.initialize_level()
 
+    def upper_level(self, init=True):
+        """Return the 'upper level' for this agent, i.e. the first host with
+        a not-None level attribute.
+
+        TAG: USER
+        """
+        if self.level is not None and not init:
+            return self
+        return self.get_host()['MASTER'].upper_level(init=False)
+
+
     def load_state_from_file(self, simu_id, filename):
         #TODO
         pass
 
     def save_state_to_file(self, simu_id, filename):
+        """Serialize the top-level agent into the filename (binary format)"""
         #TODO
+        for state_machine in self.model.state_machines.values():
+            state_machine.model = None
+        self.detach_model()
+        new_filename = '{}_{}.dat'.format(filename, simu_id)
+        print('Serialization of {} in file {}'.format(self, new_filename))
+        start = time.perf_counter()
+        with open(new_filename, 'wb') as f:
+            serializer.dump(self, f)
+        end = time.perf_counter()
+        print('Serialization finished in {:.2f} s'.format(end-start))
         pass
 
     def apply_initial_prototype(self, name=None, prototype=None, execute_actions=False):
@@ -179,8 +205,7 @@ class AbstractProcessManager(StructuredView):
         """
         if self.level in self.model.processes:
             for process in self.model.processes[self.level]:
-                if self.level not in self.model.compartments or\
-                   process not in self.model.compartments[self.level]:
+                if self.level not in self.model.compartments or process not in self.model.compartments[self.level]:
                     if process in self.model.state_machines:
                         self.add_statemachine_process(process)
                     else:
@@ -201,13 +226,14 @@ class AbstractProcessManager(StructuredView):
         followed by the evolution inherited from superclasses.
 
         """
-#        self['MASTER'].evolve()
-        for process in self:
-            process.evolve()
-        for name, process in self._content.items():
-            if name not in self.no_compart:
-                self.add_new_population(name, process.new_population)
-                process.update_counts()
+        # self['MASTER'].evolve()
+        if self.statevars._is_active:
+            for process in self:
+                process.evolve()
+            for name, process in self._content.items():
+                if name not in self.no_compart:
+                    self.add_new_population(name, process.new_population)
+                    process.update_counts()
         super().evolve(**others)
 
     def add_method_process(self, process_name, method=None):
@@ -285,6 +311,8 @@ class AbstractProcessManager(StructuredView):
             self.create_properties_for_groups(process_name, key_variables)
         # dynamically add properties for testing states in compart_class instances
         add_all_test_properties(dict_comp._content[init_key])
+        # dynamically add properties for relative counts in compart_class instances
+        add_all_relative_population_getters(dict_comp._content[init_key], key_variables)
 
     def create_count_properties_for_state(self, process_or_machine_name, state_name,
                                           count_function, aggregation_function):
@@ -364,6 +392,8 @@ class AbstractProcessManager(StructuredView):
                 'agent_id': self.agid,
                 # 'population': self.population
             })
+            if 'population_id' in self.statevars:
+                res['population_id'] = self.statevars.population_id
             if self.level in self.model.outputs and\
                'extra_vars' in self.model.outputs[self.level]:
                 for name in self.model.outputs[self.level]['extra_vars']:
@@ -395,3 +425,7 @@ class AbstractProcessManager(StructuredView):
     @property
     def population(self):
         return self.statevars.population
+
+    @abc.abstractmethod
+    def remove_all(self):
+        pass
