@@ -12,14 +12,15 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -35,24 +36,24 @@
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 import itertools                 as it
 from   collections               import OrderedDict
@@ -110,6 +111,19 @@ class CompartProcessManager(AbstractProcessManager):
             self._host = OrderedDict()
         self._host[host.keys] = host
         self.simulation = host.simulation
+        ## ADDED to correct bug when adding populations during running simulation
+        ## TODO: check if same problem with other paradigms AND if other statevars to update
+        self.statevars.step = host.statevars.step
+        ## TODO: other problem : the lines below update the step in
+        ## the group managers associated to the processes, but the
+        ## step in the counts is already 0 (in the steps after, OK) =>
+        ## see the order between step assignation and count update
+        # print("Adding host", host, "to", self)
+        for proc, comp in self._content.items():
+            if proc not in self.no_compart:
+                comp.statevars.step = self.statevars.step
+                # print(comp.statevars.step)
+        # print("host step:", host.statevars.step, " / my step", self.statevars.step)
 
     def apply_initial_conditions(self):
         """Initialize level with initial conditions specified in the model.
@@ -291,11 +305,25 @@ class CompartProcessManager(AbstractProcessManager):
         assert(agents_or_population[0] == 'population')
         self.remove_population(agents_or_population[1])
 
+
+    def remove_all(self):
+        """Remove the whole population from the current
+        CompartProcessManager.
+
+        """
+        # for process, comp in self._content.items():
+        #     if process not in self.no_compart:
+        #         # print(process, spec)
+        #         comp.remove(comp.population)
+        # self.statevars.population = 0
+        self.remove_randomly(proba=1)
+
     def remove_population(self, population_spec):
         """Remove the specified population spec from the current
         CompartProcessManager.
 
         """
+        init_pop = self.statevars.population
         self.complement_population(population_spec, remove=True)
         # print('>'*10)
         # print(population_spec)
@@ -308,12 +336,15 @@ class CompartProcessManager(AbstractProcessManager):
                 removed[process] = 0
                 for key, qty in spec.items():
                     if key in manager._content:
-                        manager[key].remove(qty)
-                        removed[process] += qty
+                        nb_removed = manager[key].remove(qty)
+                        removed[process] += nb_removed
         nb = set(removed.values())
         assert(len(nb) <= 1)
         if nb:
-            self.statevars.population -= nb.pop()
+            nb_removed = nb.pop()
+            self.statevars.population -= nb_removed
+        if init_pop < nb_removed:
+            print(init_pop, population_spec, removed,  nb_removed, self.statevars.population)
 
     def remove_randomly(self, proba=0, amount=None, process=None):
         """Remove random amounts of populations from this ProcessManager. If
@@ -369,7 +400,6 @@ class CompartProcessManager(AbstractProcessManager):
             probas = [ n / total_pop for n in pops]
             qties = np.random.multinomial(total, probas)
             to_remove[name] = dict(zip(keys, qties))
-        # print(to_remove)
         self.remove_population(to_remove)
         return to_remove
 
