@@ -12,14 +12,15 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -35,32 +36,32 @@
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 from   collections               import Counter
 
 import numpy                     as np
 import pandas                    as pd
 
-from   emulsion.tools.misc       import select_random,\
-    create_counter_getter, create_atoms_aggregator
+from   emulsion.tools.misc       import select_random
+from   emulsion.tools.getters    import create_counter_getter, create_atoms_aggregator
 
 
 
@@ -150,12 +151,13 @@ class IBMProcessManager(MultiProcessManager):
         self.new_agents.clear()
         super().evolve(**others)
         ## handle autoremove states
-        to_remove = []
-        for machine_name, state in self.autoremove_states:
-            to_remove += self.select_atoms(machine_name, value=state)
-        self.remove_atoms(set(to_remove))
-        self.add_new_population(None, self.new_agents)
-        self.update_counts()
+        if self.statevars._is_active:
+            to_remove = []
+            for machine_name, state in self.autoremove_states:
+                to_remove += self.select_atoms(machine_name, value=state)
+            self.remove_atoms(set(to_remove))
+            self.add_new_population(None, self.new_agents)
+            self.update_counts()
 
 
     def update_counts(self):
