@@ -11,14 +11,15 @@ Classes and functions for entities management.
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -34,24 +35,24 @@ Classes and functions for entities management.
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 
 from   collections               import OrderedDict
@@ -246,7 +247,8 @@ class AdaptiveView(SimpleView):
 
         """
         super().evolve(machine=machine)
-        self.check_consistency()
+        if self.statevars._is_active:
+            self.check_consistency()
 
     def check_consistency(self):
         """Check the value of the observables of the current
@@ -308,7 +310,8 @@ class StructuredView(Aggregation):
         self._notifications = {}
 
     def __iter__(self):
-        return self._content.values().__iter__()
+        # use a copy of values to allow for modifications during the time step
+        return list(self._content.values()).__iter__()
 
     def __getitem__(self, name):
         if name not in self._content: # DEBUG: should not happen !
@@ -395,7 +398,8 @@ class StructuredView(Aggregation):
 
     def evolve(self, machine=None):
         super().evolve(machine=machine)
-        self.handle_notifications()
+        if self.statevars._is_active:
+            self.handle_notifications()
 
 
 
@@ -441,7 +445,8 @@ class StructuredViewWithCounts(StructuredView):
 
     def evolve(self):
         super().evolve()
-        self.update_counts()
+        if self.statevars._is_active:
+            self.update_counts()
 
 
 
@@ -465,7 +470,7 @@ class AutoStructuredView(StructuredView):
     specific statevar as key.
 
     """
-    def __init__(self, key_variable=None, **others):
+    def __init__(self, key_variable, **others): # =None removed after Sandie's wise suggestion
         super().__init__(**others)
         self.key_variable = key_variable
         self.current_key = 0    # auto-increment ID if none specified
