@@ -12,14 +12,15 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -35,24 +36,24 @@
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 
 from   pathlib                 import Path
@@ -60,21 +61,26 @@ from   datetime                import datetime, timedelta
 from   copy                    import deepcopy
 
 import yaml
+import numpy                   as     np
 import dateutil.parser         as     dup
 
 from   sympy                   import sympify, Symbol
 
+from   collections             import OrderedDict
 from   sortedcontainers        import SortedSet, SortedDict
 from   jinja2                  import Environment, PackageLoader
 
 import networkx                as     nx
 
 from   emulsion.tools.state    import StateVarDict
-from   emulsion.tools.misc     import read_from_file, load_class, create_new_serial, create_weighted_random
+from   emulsion.tools.misc     import read_csv_prototypes, read_from_file, load_class
+from   emulsion.tools.getters  import create_new_serial, create_weighted_random,\
+    create_successor_getter, create_predecessor_getter,\
+    make_random_prototype_getter, make_generator_prototype_getter
 from   emulsion.tools.calendar import EventCalendar
 
 from   emulsion.model.functions      import DEFAULT_LEVEL_INFO, DEFAULT_GROUPING_INFO,\
-    make_function, make_when_condition
+    make_function, make_CSV_function, make_when_condition, make_statevar_getter, make_model_value_getter
 from   emulsion.model.exceptions     import SemanticException
 from   emulsion.model.state_machines import StateMachine
 
@@ -98,12 +104,13 @@ class EmulsionModel(object):
     etc.
 
     """
-    def __init__(self, filename=None, description=None):
+    def __init__(self, filename=None, description=None, input_dir=None):
         """Instantiate the model either from a configuration file or
         from a string. Both must contain a YAML-based description of
         the model.
 
         """
+        self.input_dir = input_dir
         if filename:
             self.filename = filename
             self.parse(read_from_file(filename))
@@ -164,6 +171,8 @@ class EmulsionModel(object):
         self._events = {}
         # dict of levels
         self.levels = {}
+        # dict of default prototypes associated to levels
+        self.default_prototypes = {}
         # dict of aggregate vars by level
         self.aggregate_vars = {}
         # list of the processes involved in the model
@@ -186,6 +195,9 @@ class EmulsionModel(object):
         self.distributions = {}
         # dict of all prototypes encountered in the model
         self.prototypes = {}
+        # prototype collections: list of functions providing a prototype_name
+        # for a given collection name
+        self.prototype_collection = {}
         # dict of all initial conditions encountered in the model
         self.initial_conditions = {}
         # dict of enumerate types used in special variables
@@ -587,55 +599,145 @@ class EmulsionModel(object):
 
         Example of YAML specification:
         ------------------------------
-        prototypes:
-          animals:
-            - newborn:
-                desc: prototype for new calves
-                health_state: M
-                life_state: NP
-                age: 0
+
+        .. code-block:: yaml
+
+           prototypes:
+             animals:           # name of a level
+               - newborn:       # prototype name
+                   # description of the prototype
+                   desc: 'prototype for new calves'
+                   # list of variables with values
+                   health_state: M
+                   life_state: NP
+                   age: 0
 
         """
         if 'prototypes' in self._description:
             for level, prototypes in self._description['prototypes'].items():
                 self.prototypes[level] = {}
                 for list_item in prototypes:
+                    # read all prototypes for a given level
                     for (key, val) in list_item.items():
-                        self.prototypes[level][key] = {}
+                        prototype = OrderedDict()
+                        # read all variables for prototype named "key"
                         for variable, value in val.items():
-                            result = value
-                            if variable != 'desc':
-                                if value == 'random':
-                                    result = '_random_' + variable
-                                elif value == 'default':
-                                    result = '_default_' + variable
-                                elif value == 'next_state':
-                                    result = '_next_state_' + variable
-                                    # associate to a function that returns the successor of the state contained in the variable (or a random state if the variable was not defined)
-                                    self._values[result] = lambda agent: agent.statevars[variable].successor if variable in agent.statevars else agent.model.state_machines[variable].get_random_state()
-                                elif value == 'previous_state':
-                                    result = '_previous_state_' + variable
-                                    # associate to a function that returns the predecessor of the state contained in the variable (or a random state if the variable was not defined)
-                                    self._values[result] = lambda agent: agent.statevars[variable].predecessor if variable in agent.statevars else agent.model.state_machines[variable].get_random_state()
-                                elif type(value) == str and value.startswith('random('):
-                                    # result = '_weighted_random_' + variable
-                                    result = '{}_{}'.format(variable, value)
-                                    if value[6] == '(' and value[-1] == ')':
-                                        weights = [self.add_expression(e) for e in value[7:-1].split(',')]
-                                    self._values[result] = create_weighted_random(variable, weights, model=self)
-                                elif type(value) == str and value.startswith('serial'):
-                                    result = '_serial_' + variable
-                                    if value != 'serial' and value[6] == '(' and value[-1] == ')':
-                                        end = self.add_expression(value[7:-1])
-                                    else:
-                                        start, end = 0, None
-                                    self._values[result] = create_new_serial(end=end, model=self)
-                                elif value in self.statevars:
-                                    self._values[result] = lambda agent: agent.statevars[result]
-                                self.prototypes[level][key][variable] = result
+                            result = self.parse_prototype_line(variable, value)
+                            if result is not None:
+                                prototype[variable] = result
+                        # test if an ordered affectation is defined:
+                        # if so, parse the list of variable
+                        # assignation as a list of tuples (name of
+                        # variable, value)
+                        for keyword in ['begin_with', 'end_with']:
+                            if keyword in val:
+                                l_items = []
+                                for proto_item in val[keyword]:
+                                    for variable, value in proto_item.items():
+                                        result = self.parse_prototype_line(variable, value)
+                                        l_items.append((variable, result))
+                                prototype[keyword] = l_items
+                        # test if a collection is defined
+                        if 'file' in val:
+                            # this is not an isolated prototype but a
+                            # collection of prototypes specified in a
+                            # CSV file
+                            # check if any filtering condition on the lines of the CSV file
+                            cond = make_CSV_function(self.expand_expression(val['filter']), self.modules) if 'filter' in val else None
+                            include_values = val['include_values'] if 'include_values' in val else []
+                            exclude_values = val['exclude_values'] if 'exclude_values' in val else []
+                            # check the selection method that will be used to pick real prototypes in the collection
+                            if 'select' not in val:
+                                raise SemanticException('No selection method provided for prototype collection {}'.format(key))
+                            method = val['select']
+                            par_start = method.find('(')
+                            if par_start > 0:
+                                par_end = method.find(')')
+                                if par_end < par_start:
+                                    raise SemanticException('Bad parenthesis in selection method {} for prototype collection {}'.format(method, key))
+                                special_column = method[(par_start+1):par_end]
+                                method = method[0:par_start]
+                            else:
+                                special_column = None
+                            if method not in ['ordered', 'ordered_cycle', 'random_replace', 'random_noreplace']:
+                                raise SemanticException('Unknown selection method {} for prototype collection {}'.format(method, key))
+                            collection = read_csv_prototypes(key, self.input_dir.joinpath(val['file']), method=method,
+                                                             condition=cond, weight_column=special_column, include=include_values, exclude=exclude_values)
+                            for proto_freq, proto_name, proto_desc in collection:
+                                specific_prototype = OrderedDict()
+                                # parse each
+                                for spec_var, spec_val in proto_desc.items():
+                                    result = self.parse_prototype_line(spec_var, spec_val)
+                                    if result is not None:
+                                        specific_prototype[spec_var] = result
+                                # correct prototype defined in CSV
+                                # file with information provided in
+                                # YAML model
+                                specific_prototype.update(prototype)
+                                # add each concrete prototype
+                                self.prototypes[level][proto_name] = specific_prototype
+                                # print("Added {}: {}".format(proto_name, specific_prototype))
+                            # register collection named "key"
+                            freqs, names, _ = zip(*collection)
+                            if method == 'random_replace':
+                                self.prototype_collection[key] = make_random_prototype_getter(names, freqs)
+                            else:
+                                self.prototype_collection[key] = make_generator_prototype_getter(names, freqs, key, method, (method == 'ordered_cycle'))
+                            # print(key, "->", self.prototype_collection[key])
+                        else:
+                            self.prototypes[level][key] = prototype
 
 
-    def get_prototype(self, level, name):
+    def parse_prototype_line(self, variable, value):
+        """Parse a line in a prototype definition, udpate the model if needed,
+        and return the resulting value.
+
+        """
+        if variable in ['desc', 'begin_with', 'end_with', 'file', 'filter', 'select', 'include_values', 'exclude_values']:
+            return None
+        # default case
+        result = value
+        if value == 'random':
+            result = '_random_' + variable
+        elif value == 'default':
+            result = '_default_' + variable
+        elif value == 'next_state':
+            result = '_next_state_' + variable
+            # associate to a function that returns the successor of
+            # the state contained in the variable (or a random state
+            # if the variable was not defined)
+            self._values[result] = create_successor_getter(variable)
+        elif value == 'previous_state':
+            result = '_previous_state_' + variable
+            # associate to a function that returns the predecessor of
+            # the state contained in the variable (or a random state
+            # if the variable was not defined)
+            self._values[result] = create_predecessor_getter(variable)
+        elif type(value) == str and value.startswith('random('):
+            # result = '_weighted_random_' + variable
+            result = '{}_{}'.format(variable, value)
+            if value[6] == '(' and value[-1] == ')':
+                weights = [self.add_expression(e) for e in value[7:-1].split(',')]
+            self._values[result] = create_weighted_random(variable, weights, model=self)
+        elif type(value) == str and value.startswith('serial'):
+            result = '_serial_' + variable
+            if value != 'serial' and value[6] == '(' and value[-1] == ')':
+                end = self.add_expression(value[7:-1])
+            else:
+                start, end = 0, None
+            self._values[result] = create_new_serial(end=end, model=self)
+        elif isinstance(value, list):
+            result = tuple(value)
+        elif value in self.statevars:
+            self._values[result] = make_statevar_getter(result)
+        elif value in self.parameters:
+            self._values[result] = make_model_value_getter(result)
+        elif value not in self.states:
+            self._values[value] = self.add_expression(value)
+        return result
+
+
+    def get_prototype(self, level, name, simu_id=None):
         """Return a ready-to-use prototype, i.e. a StateVarDict corresponding
         to the prototype for the specified level and name, where
         symbols associated to statevariables are already replaced by
@@ -643,10 +745,25 @@ class EmulsionModel(object):
         into tuples (to be hashable).
 
         """
-        return StateVarDict({
-            var: tuple(val) if isinstance(val, list) else self.get_value(val)
-            for var, val in self.prototypes[level][name].items()
-        })
+        if name in self.prototype_collection:
+            # use the associated function to retrieve concrete prototype
+            prototype_name = self.prototype_collection[name](simu_id)
+            # print(prototype_name)
+            # print(self.prototypes[level][prototype_name])
+        else:
+            prototype_name = name
+        ## compute concrete prototype by getting relevant values
+        result = StateVarDict()
+        for var, val in self.prototypes[level][prototype_name].items():
+            if var in ['begin_with', 'end_with']:
+                ordered = []
+                for k, v in val:
+                    value = tuple(v) if isinstance(v, list) else self.get_value(v)
+                    ordered.append((k, value))
+                result[var] = ordered
+            else:
+                result[var] = tuple(val) if isinstance(val, list) else self.get_value(val)
+        return result
 
     def build_levels(self):
         """Parse the description of different level of simulations.
@@ -731,6 +848,8 @@ class EmulsionModel(object):
                             if 'module' not in desc:
                                 desc['module'] = None
                                 desc['class_name'] = None
+            if 'default_prototype' in desc:
+                self.default_prototypes[level] = desc['default_prototype']
             if 'aggregation_type' in desc:
                 default = DEFAULT_LEVEL_INFO[desc['aggregation_type']]
                 if 'super' not in desc:
