@@ -12,14 +12,15 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -35,26 +36,27 @@
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 
+import sys
 from   functools               import partial
 
 import numpy                   as     np
@@ -148,6 +150,9 @@ class StateMachine(object):
                 # provide a default fillcolor
                 if 'fillcolor' not in value:
                     value['fillcolor'] = 'lightgray'
+                # provide a default dashed
+                if 'line_style' not in value:
+                    value['line_style'] = 'solid'
                 # if properties are provided, add the corresponding
                 # expression to the model
                 if 'properties' not in value:
@@ -182,8 +187,13 @@ class StateMachine(object):
                     if keyword in value:
                         self._add_state_actions(name, keyword, value[keyword])
         # build the enumeration of the states
+        # print('*'*50)
+        # print(dir(sys.modules[__name__]))
         self.states = EmulsionEnum(self.machine_name.capitalize(),
                                    states, module=__name__)
+        # print(sys.modules[__name__], self.states.__name__, self.states)
+        setattr(sys.modules[__name__], self.states.__name__, self.states)
+        # print(dir(sys.modules[__name__]))
         # link the states to their state machine
         self.states.state_machine = self
         # define the default value for "autoremove" at the enumeration level
@@ -270,6 +280,13 @@ class StateMachine(object):
                 for state in self.states
                 if not state.autoremove}
 
+    @property
+    def state_style(self):
+        """Return a dictionary of state names associated with dashed state."""
+        return {state.name: self._statedesc[state.name]['line_style']
+                for state in self.states
+                if not state.autoremove}
+
 
     def build_graph(self):
         """Parse the description of the state machine and extract the
@@ -547,7 +564,7 @@ class StateMachine(object):
                             )
                         )
                         understood = True
-                for keyword in ['set_var']:
+                for keyword in ['set_var', 'set_upper_var']:
                     if keyword in d_action:
                         # assume that increase statevar with rate
                         l_actions.append(
@@ -651,7 +668,7 @@ class StateMachine(object):
                 if onenter or onstay or onexit:
                     actions = 'shape="Mrecord", label="{%s{\ %s\ %s}%s}", ' % (
                         onenter, name_lab, onstay, onexit)
-            output += '\t"{}" [{}tooltip="{}", fillcolor={}, style="{}"] ;\n'.format(
+            output += '\t"{}" [{}tooltip="{}", fillcolor="{}", style="{}"] ;\n'.format(
                 name, actions,
                 self._statedesc[name]['tooltip'],
                 # '\n\tON ENTER: {}'.format(self.state_actions[name]['on_enter'])\
