@@ -12,14 +12,15 @@ especially regarding introspection.
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -35,36 +36,41 @@ especially regarding introspection.
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 
 from   functools                 import partial
 from   importlib                 import import_module
 from   typing                    import List, Iterable, Tuple, Union, Any
+from   collections               import OrderedDict
+from   operator                  import itemgetter
+import itertools                 as it
 
+import csv
 import yaml
 
 import numpy                     as np
 
 from   sortedcontainers          import SortedSet
-
+from   emulsion.model.exceptions import SemanticException
+from   emulsion.tools.getters    import create_duration_getter, create_state_tester, create_relative_population_getter
 
 AGENTS = 1
 """Constant value for the index where list of agents are stored in
@@ -404,8 +410,8 @@ def add_new_property(agent, property_name, getter_function):
         the agent to which the property must be added
     property_name: str
         the name of the property
-    getter_function: lambda
-        the function upon which the property is built
+    getter_function: callable
+        the callable class upon which the property is built
 
     """
     setattr(agent.__class__, property_name, property(getter_function))
@@ -413,48 +419,11 @@ def add_new_property(agent, property_name, getter_function):
     # print('Agent {} adding property {} to class {}'.format(agent, property_name, agent.__class__))
 
 
-def create_population_getter(process_name: str,
-                             group_name: Union[str, Tuple]):
-    """Build a getter for property of the form ``total_X_Y`` where ``X``
-    and ``Y`` are states of two different states machines used in a
-    grouping.
-
-    The functions can handle groupings with an arbitrary number of states.
-
-    Parameters
-    ----------
-    process_name : str
-        the name of the process associated with the grouping for which
-        the population getter is created
-    group_name : str or tuple
-        either a string representing the state name (if only one), or
-        a tuple of several state names
-
-    Returns
-    -------
-    callable :
-        A function which can be applied to an `AbstractProcessManager`
-        agent which returns the population size for the specified
-        group.
+def add_all_test_properties(agent):
+    """Endow the agent with individuals properties: ``duration_in_<a state
+    machine>`` for all state machines, and ``is_X`` for all states.
 
     """
-    if not isinstance(group_name, tuple):
-        group_name = (group_name,)
-    # print('adding automatic property for', process_name, group_name)
-    return lambda self: self.get_group_population(process_name, group_name)
-
-def create_counter_getter(machine_name, state_name):
-    return lambda self: self.counters[machine_name][state_name]
-
-
-def create_state_tester(state_name):
-    return lambda self: self.is_in_state(state_name)
-
-def create_duration_getter(machine_name):
-    # print('adding automatic property for', process_name, group_name)
-    return lambda self: self.duration_in_current_state(machine_name)
-
-def add_all_test_properties(agent):
     for machine in agent.model.state_machines.values():
         add_new_property(agent, 'duration_in_{}'.format(machine.machine_name),
                          create_duration_getter(machine.machine_name))
@@ -462,147 +431,40 @@ def add_all_test_properties(agent):
             add_new_property(agent, 'is_{}'.format(state.name),
                              create_state_tester(state.name))
 
-
-def find_operator(operator: str):
-    """Return an aggregation function named *operator*.
-
-    Search starts with emulsion functions (module
-    `emulsion.tools.functions`), which includes Python built-ins, then
-    in `numpy`.
-
-    A special shortcut is provided for percentiles: ``percentileXX``
-    is interpreted as the partial function
-    ``numpy.percentile(q=int(XX))``.
+def add_all_relative_population_getters(agent, key_variables):
+    """Endow the agent with individual properties of the form ``total_X_Y``
+    where ``X`` and ``Y`` are either states of two different states
+    machines used in a grouping, or keywords built upon a state
+    machine name ``M`` of the form ``my_M`` or ``other_M``.
 
     Parameters
     ----------
-    operator: str
-        the name of the aggregation operator
-
-    Returns
-    -------
-    lambda:
-        a function that takes a list (or array-like) as input and
-        returns the application of the *operator* to the values.
-
+    agent: EmulsionAgent (AdaptiveView or AtomAgent)
+        the agent which will be endowed with the new properties
+    key_variables: tuple
+        string of tuple containing the names of the state machines
     """
-    op = None
-    current_module = import_module('emulsion.tools.functions')
-    # search in emulsion functions module first
-    if hasattr(current_module, operator):
-        op = getattr(current_module, operator)
-    elif operator.startswith('percentile'):
-        # special case of 'percentileXX' where XX is a number (0 to
-        # 100) and automatically extracted
-        op = partial(np.percentile, q=int(operator[10:]))
-    else:
-        # otherwise search in numpy
-        op = getattr(np, operator)
-    return op
+    # compute all relevant combinations between states of the state
+    # machines and 'my_statemachine', 'other_statemachine'
+    all_combinations = set(it.product(
+        *[[state.name
+           for state in agent.model.state_machines[machine_name].states
+           if not state.autoremove] +\
+          ['my_{}'.format(machine_name), 'other_{}'.format(machine_name)]
+          for machine_name in key_variables]))
+    state_combinations = set(it.product(
+        *[[state.name
+           for state in agent.model.state_machines[machine_name].states
+           if not state.autoremove]
+          for machine_name in key_variables]))
+    combinations = all_combinations - state_combinations
+    # add corresponding properties
+    for combination in combinations:
+        prop_name = 'total_{}'.format('_'.join(combination))
+        add_new_property(agent, prop_name,
+                         create_relative_population_getter(agent.model, combination))
+        # print('Added', prop_name, 'to agent', agent)
 
-def create_aggregator(sourcevar: str, operator: str):
-    """Create an aggregator function to be used as property getter. This
-    aggregator function has to collect all values of *sourcevar* for
-    agents contained in a given host, and reduce them to one avalue
-    using the specified *operator*.
-
-    Parameters
-    ----------
-    sourcevar: str
-        the name of the variable to collect in the sublevel
-    operator: str
-        the name of the operator to apply to the collected values
-
-    Returns
-    -------
-    lambda:
-        A function which can be applied to a `MultiProcessManager`
-        agent (i.e. with explicit sublevel agents) which returns the
-        aggregated values for the whole population.
-
-    """
-    op = find_operator(operator)
-    return lambda self: op([sublevel.get_information(sourcevar)
-                            for sublevel in self['MASTER']])
-
-def create_group_aggregator(sourcevar: str, operator: str, process_name: str,
-                            group_name: Union[str, Tuple]):
-    """Build a getter for property of the form ``newvar_X_Y`` where ``X``
-    and ``Y`` are states of two different states machines used in a
-    grouping, ``newvar`` is an aggregate variable based on collecting
-    all values of *sourcevar* for the specific *group_name* and
-    aggregating the values using *operator*. The functions can handle
-    groupings with an arbitrary number of states.
-
-    Parameters
-    ----------
-    sourcevar: str
-        the name of the variable to collect in the sublevel
-    operator: str
-        the name of the operator to apply to the collected values
-    process_name: str
-        the name of the process associated with the grouping for which
-        the population getter is created
-    group_name: str or tuple
-        either a string representing the state name (if only one), or
-        a tuple of several state names
-
-    Returns
-    -------
-    lambda:
-        A function which can be applied to an `MultiProcessManager`
-        agent (i.e. with explicit sublevel agents) which returns the
-        aggregated value for the specified group.
-
-    """
-    op = find_operator(operator)
-    if not isinstance(group_name, tuple):
-        group_name = (group_name,)
-    # print('adding automatic property for', process_name, group_name)
-    def aggregate_group(self):
-        try:
-            return op([sublevel.get_information(sourcevar)
-                       for sublevel in self.get_group_atoms(process_name, group_name)])
-        except ValueError:
-            return np.nan
-    return aggregate_group
-
-def create_atoms_aggregator(sourcevar: str, operator: str,
-                            machine_name: str, state_name: str):
-    """Build a getter for property of the form ``newvar_X`` where ``X`` if
-    the value of *state_name* (a state of *state_machine*), ``newvar``
-    is an aggregate variable based on collecting all values of
-    *sourcevar* for the specific *group_name* and aggregating the
-    values using *operator*. This function is intended to work on
-    `IBMProcessManager` agents, which do not benefit from groupings.
-
-    Parameters
-    ----------
-    sourcevar: str
-        the name of the variable to collect in the sublevel
-    operator: str
-        the name of the operator to apply to the collected values
-    machine_name: str
-        the name of the state machine for which the getter is created
-    state_name: str
-        the state name
-
-    Returns
-    -------
-    lambda:
-        A function which can be applied to an `IBMProcessManager`
-        agent (i.e. with explicit but ungrouped sublevel agents) which
-        returns the aggregated value for the specified group.
-
-    """
-    op = find_operator(operator)
-    def aggregate_atoms(self):
-        try:
-            return op([sublevel.get_information(sourcevar)
-                            for sublevel in self.select_atoms(machine_name, state=state_name)])
-        except ValueError:
-            return np.nan
-    return aggregate_atoms
 
 def serial(start=0, end=None, model=None):
     """A very simple serial number generator."""
@@ -615,52 +477,100 @@ def serial(start=0, end=None, model=None):
                 value = start
 
 
-def create_new_serial(end=None, model=None):
-    """Create the serial number generator associated to the specified
-    variable.
+def read_csv_prototypes(collection_name, filename, method=None, condition=None, weight_column=None, include=[], exclude=[]):
+    """Read the definitions of prototypes contained in file *filename* and
+    return a dictionary representing the collection.
 
-    """
-    generator = serial(start=0, end=end, model=model)
-    return lambda self: next(generator)
-
-
-def create_weighted_random(machine_name, weights, model=None):
-    """Create a random choice function which returns a random state from
-    the given state machine (among non-autoremove states), according
-    to the weights. Weights are interpreted either directly as
-    probabilities (if the number of weights is stricly one below the
-    number of available states, the last state getting the complement
-    to 1), or as true weights which are then normalized to be used as
-    probabilities.
-
-    Parameters:
-    -----------
-    machine_name: str
-        the name of the state machine where the states must be chosen
-        among the *N* non-autoremove states.
-    weights: list
-        a list of *N* or *N-1* model expressions assumed to produce positive numbers
-    model: EmulsionModel
-        the model where this function is defined
+    Parameters
+    ----------
+    collection_name: str
+        the name of the prototypes collection
+    filename: Path
+        the path to the CSV file to read
+    method: str
+        method used to select prototypes in the collection
+    condition: callable
+        a callable class which can be called on a CSV line, returning
+        1 if the line fulfils the condition, 0 otherwise
+    weight_column: str
+        the name of the column to use to weight prototypes in random
+        sampling. If None, all prototypes are given the same weight
 
     Returns
     -------
-    lambda: a function that returns a random state according to the
-        values of the weights list, interpreted either as
-        probabilities (if size *N-1*) or as weights (if size *N*)
-        which are then normalized to provide probabilities
+    list:
+        A prototype collection, as a list of tuples composed of the
+        prototype frequency, the prototype name, and the dictionary
+        describing the prototype itself
+
+    Notes
+    -----
+    The CSV file is expected to contain a description of the
+    prototypes. Prototypes define how to set variables in agents that
+    belong to the same organization level. The CSV header is expected
+    to contain the names of the variables. A column named
+    *prototype_name* can be used to associate each combination of
+    variable values to a specific identifier (built from the
+    collection name followed by the prototype name, e.g. collection
+    `default_herd` and *prototype_name* `small` gives
+    `default_herd-small`). If not specified, the prototype names will
+    be generated automatically based on the filename and line
+    numbers. The resulting prototypes are named after this initial
+    prototype name (automatically generated or specified) and the
+    collection name (`collection_name-prototype_name`).  A *filter*
+    can be used in the prototype section to select only lines that
+    fulfil a given condition (provided as a function here).
 
     """
-    machine = model.state_machines[machine_name]
-    states = [state for state in machine.states if not state.autoremove]
-    assert(len(states) - len(weights) <= 1)
-    def weighted_random(agent):
-        values = [agent.get_model_value(weight) for weight in weights]
-        total = sum(values)
-        if len(values) < len(states):
-            assert(0 <= total <= 1)
-            probas = values + [1 - total]
-        else:
-            probas = [v / total for v in values]
-        return np.random.choice(states, p=probas)
-    return weighted_random
+    names = []
+    freqs = []
+    protos = []
+    default_proto_name = filename.stem
+
+    with open(filename, 'r') as csvfile:
+        csvreader = csv.DictReader(csvfile, delimiter=',')
+        fields = csvreader.fieldnames
+        if weight_column is not None and weight_column not in fields:
+            raise SemanticException('Column {} used to weight prototypes not found in file {}.format(weight_column, filename)')
+        variables = [f for f in fields if f != 'prototype_name']
+        for line_number,  row in enumerate(csvreader):
+            # manage explicit list of inclusion values
+            # (row kept only if at least one value in the row is in inclusions list)
+            if include:
+               if not any(v in include for v in row.values()):
+                   continue
+            # manage explicit list of exclusion values
+            # (row discarded if at least one value in the row is in the exclusion list)
+            if exclude:
+                if any(v in exclude for v in row.values()):
+                    continue
+            # manage filtering conditions
+            if condition is not None:
+                if not condition(row):
+                    continue
+            # if prototype_name is given, use it
+            if 'prototype_name' in fields:
+                names.append('{}-{}'.format(collection_name, row['prototype_name']))
+                del row['prototype_name']
+            else:
+                # otherwise generate prototype name from file name and line number
+                names.append('{}-{}{}'.format(collection_name, default_proto_name, line_number))
+            # if prototype_frequency provided, use it
+            if weight_column is not None:
+                freqs.append(float(row[weight_column]))
+            else:
+                # otherwise consider all prototypes equiprobable
+                freqs.append(1)
+            #
+            proto = OrderedDict([(key, row[key]) for key in variables])
+            protos.append(proto)
+    # normalize frequencies
+    s = sum(freqs)
+    freqs = [ freq / s for freq in freqs]
+    result = list(zip(freqs, names, protos))
+    if method != 'random_replace':
+        # if method is 'ordered', sort prototypes according to weight_column
+        if method.startswith('ordered'):
+            result = sorted(result, key=itemgetter(0))
+    # print(result)
+    return result
