@@ -11,14 +11,15 @@ Plotting tools... to be improved!
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inrae.fr)
+#     - Sébastien Picault (sebastien.picault@inra.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
 # 
-#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
+#     BIOEPAR, INRAE, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
 # 
 # 
 # How to cite:
@@ -34,24 +35,24 @@ Plotting tools... to be improved!
 # License:
 # --------
 # 
-#    Copyright 2016 INRAE and Univ. Lille
+#     Copyright 2016 INRAE and Univ. Lille
 # 
-#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+#     Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
-#    Agence pour la Protection des Programmes,
-#    54 rue de Paradis, 75010 Paris, France
+#     Agence pour la Protection des Programmes,
+#     54 rue de Paradis, 75010 Paris, France
 # 
-#    Licensed under the Apache License, Version 2.0 (the "License");
-#    you may not use this file except in compliance with the License.
-#    You may obtain a copy of the License at
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 # 
-#        http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 # 
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS,
-#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    See the License for the specific language governing permissions and
-#    limitations under the License.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 
 from   pathlib                      import Path
 
@@ -72,7 +73,7 @@ from   bokeh.palettes               import all_palettes
 
 
 def build_state_plot(counts, cols, machine, model, y='quantity', group='state',
-                     ylab='Number of individuals'):
+                     ylab='Number of individuals', style=None):
     """Return a bokeh figure based on the representation of the states in
     the *counts* dataframe, with the associated colors.
 
@@ -98,12 +99,12 @@ def build_state_plot(counts, cols, machine, model, y='quantity', group='state',
     leg_pos, leg_orient = 'right', 'vertical'
     metapop = 'population_id' in counts.columns
     if metapop:
-        max_pop_id = 1 + max(counts.population_id)
+        pop_IDs = np.sort(np.unique(counts.population_id.values))
         ncols = 2
         WIDTH = 1200
     else:
-        max_pop_id = 1
-    for pop_id in range(max_pop_id):
+        pop_IDs = [0]
+    for pop_id in pop_IDs:
         if metapop:
             df = counts[counts.population_id == pop_id]
         else:
@@ -117,14 +118,15 @@ def build_state_plot(counts, cols, machine, model, y='quantity', group='state',
         legend_items = []
         for state in cols:
             state_items = []
+            line_style = style[state] if style is not None else 'solid'
             subdf = df[df[group] == state]
             if NB_SIMU <= 50:   # don't draw points if too many !
                 ci = plot.circle(x=time_unit, y='quantity', source=ColumnDataSource(subdf),
                                  size=2, color=cols[state], alpha=alpha)
                 state_items.append(ci)
-            for simu_id in range(1+max(df.simu_id)):
+            for simu_id in range(int(1+df['simu_id'].max())):
                 src = ColumnDataSource(subdf[subdf.simu_id == simu_id])
-                pl = plot.line(x=time_unit, y='quantity', source=src, color=cols[state], alpha=alpha, line_width=2)
+                pl = plot.line(x=time_unit, y='quantity', source=src, color=cols[state], alpha=alpha, line_width=2, line_dash=line_style)
                 state_items.append(pl)
             legend_items.append((state, state_items))
         plot.title.text = 'Evolution of %s (%s)' % (machine.replace('_', ' '), model.model_name)
@@ -215,12 +217,13 @@ def plot_outputs(params):
         if params.view_machines:
             all_figs.append(build_machine_plot(sm_name, model, params))
         col_dict = state_machine.state_colors
+        style_dict = state_machine.state_style
         # col_dict.update(population="black")
         states, cols = zip(*sorted(col_dict.items()))
         vals += [state.name for state in state_machine.states]
         df2 = pd.melt(df, id_vars=vars, value_vars=states,
                       var_name='state', value_name='quantity')
-        fig = build_state_plot(df2, col_dict, sm_name, model)
+        fig = build_state_plot(df2, col_dict, sm_name, model, style=style_dict)
         all_figs.append(fig)
         # figpath = Path(params.figure_dir,
         #                model.model_name + '_' + sm_name + '.png')
