@@ -8,6 +8,7 @@ Tools for providing generic simulation classes.
 
 
 import abc
+
 from   pathlib        import Path
 from   os.path        import exists
 from   typing         import List, Iterable, Tuple, Union
@@ -19,7 +20,7 @@ from   tqdm           import trange, tqdm   # a nice replacement for progressbar
 
 from   sqlalchemy     import create_engine
 
-from   emulsion.model import EmulsionModel
+from   emulsion.model.emulsion_model import EmulsionModel
 
 #   ____        _               _   __  __
 #  / __ \      | |             | | |  \/  |
@@ -144,8 +145,7 @@ class AbstractSimulation(object):
         """
         # ID of simulation
         self.start_id = start_id
-        self.model = EmulsionModel(filename=model_path) if model is None else model
-        self.model.input_dir = input_dir
+        self.model = EmulsionModel(filename=model_path, input_dir=input_dir) if model is None else model
         self.stock_agent = stock_agent
         self.target_agent_class = target_agent_class
         self.output_manager = OutputManager(model=self.model, output_dir=output_dir)
@@ -201,7 +201,8 @@ class Simulation(AbstractSimulation):
 
     """
     def __init__(self, steps: int = 100, simu_id: int = 0,
-                 silent: bool = False, quiet: bool = False, **others):
+                 silent: bool = False, quiet: bool = False,
+                 nocount: bool = False, **others):
         """Create an instance of simulation.
 
         Args:
@@ -218,6 +219,7 @@ class Simulation(AbstractSimulation):
         self.steps = steps
         self.silent = silent
         self.quiet = quiet
+        self.nocount = nocount
 
         del(others['target_agent_class'])
         others['simu_id'] = self.simu_id
@@ -257,10 +259,11 @@ class Simulation(AbstractSimulation):
         else:
             progress = trange(self.steps, desc=f'[Run {self.simu_id}]',
                               bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt}')
-        self.update_csv_counts(self.counts, dparams=dparams)
+        if not self.nocount:
+            self.update_csv_counts(self.counts, dparams=dparams)
         for step in progress:
             self.agent.evolve()
-            if step % self.outputs_period == 0:
+            if step % self.outputs_period == 0 and not self.nocount:
                 self.update_csv_counts(self.counts,
                                        # header=not exists(self.counts_path),
                                        dparams=dparams)
@@ -315,7 +318,8 @@ class MultiSimulation(AbstractSimulation):
     """
     def __init__(self, multi_id: int = 0, nb_simu: int = 100,
                  set_seed: bool = False, silent: bool = False,
-                 quiet: bool = False, dparams: dict = {}, **others):
+                 quiet: bool = False, nocount: bool = False,
+                 dparams: dict = {}, **others):
         """Initialize a simulation with multiple repetitions of the same
         model.
         """
@@ -326,6 +330,7 @@ class MultiSimulation(AbstractSimulation):
         self.others = others
         self.silent = silent
         self.quiet = quiet
+        self.nocount = nocount
         self.dparams = dparams
 
         self.d_simu = dict()
@@ -352,8 +357,8 @@ class MultiSimulation(AbstractSimulation):
             if 'model' not in self.others:
                 self.others['model'] = self.model
             simu = Simulation(simu_id=simu_id, silent=self.silent,
-                              quiet=self.quiet, dparams=self.dparams,
-                              **self.others)
+                              quiet=self.quiet, nocount=self.nocount,
+                              dparams=self.dparams, **self.others)
 
             simu.run()
 
